@@ -1,17 +1,16 @@
 from __future__ import annotations
 
-from typing import Tuple, Union, overload
+from typing import Sequence, Tuple, TypeVar, overload
 
 import numpy as np
-from sklearn.utils import check_random_state
 from typing_extensions import Literal
 
-from ... import concatenate
-from ..._utils import RandomStateLike
-from ...datasets import make_gaussian_process
+from ...datasets import make_gaussian
 from ...misc.metrics import lp_distance
-from ...representation import FData, FDataGrid
-from ...representation._typing import ArrayLike
+from ...misc.validation import validate_random_state
+from ...representation import FData, FDataGrid, concatenate
+from ...typing._base import RandomStateLike
+from ...typing._numpy import ArrayLike, NDArrayFloat
 
 
 def v_sample_stat(fd: FData, weights: ArrayLike, p: int = 2) -> float:
@@ -92,7 +91,36 @@ def v_sample_stat(fd: FData, weights: ArrayLike, p: int = 2) -> float:
     ))
 
 
-def v_asymptotic_stat(fd: FData, weights: ArrayLike, p: int = 2) -> float:
+def _v_asymptotic_stat_with_reps(
+    *fds: FData,
+    weights: ArrayLike,
+    p: int = 2,
+) -> NDArrayFloat:
+    """Vectorized version of v_asymptotic_stat for repetitions."""
+    weights = np.asarray(weights)
+    if len(weights) != len(fds):
+        raise ValueError("Number of weights must match number of groups.")
+    if np.count_nonzero(weights) != len(weights):
+        raise ValueError("All weights must be non-zero.")
+
+    t_ind = np.tril_indices(len(fds), -1)
+
+    results = np.zeros(shape=(len(t_ind[0]), fds[0].n_samples))
+    for i, pair in enumerate(zip(*t_ind)):
+        left_fd = fds[pair[1]]
+        coef = np.sqrt(weights[pair[1]] / weights[pair[0]])
+        right_fd = fds[pair[0]] * coef
+        results[i] = lp_distance(left_fd, right_fd, p=p) ** p
+
+    return np.sum(results, axis=0)  # type: ignore[no-any-return]
+
+
+def v_asymptotic_stat(
+    fd: FData,
+    *,
+    weights: ArrayLike,
+    p: int = 2,
+) -> float:
     r"""
     Compute asymptitic statistic.
 
@@ -146,35 +174,23 @@ def v_asymptotic_stat(fd: FData, weights: ArrayLike, p: int = 2) -> float:
 
         Finally the value of the statistic is calculated:
 
-        >>> v_asymptotic_stat(fd, weights)
+        >>> v_asymptotic_stat(fd, weights=weights)
         0.0018159320335885969
 
     References:
         .. footbibliography::
 
     """
-    weights = np.asarray(weights)
-    if not isinstance(fd, FData):
-        raise ValueError("Argument type must inherit FData.")
-    if len(weights) != fd.n_samples:
-        raise ValueError("Number of weights must match number of samples.")
-    if np.count_nonzero(weights) != len(weights):
-        raise ValueError("All weights must be non-zero.")
-
-    t_ind = np.tril_indices(fd.n_samples, -1)
-    coef = np.sqrt(weights[t_ind[1]] / weights[t_ind[0]])
-    left_fd = fd[t_ind[1]]
-    right_fd = fd[t_ind[0]] * coef
-    return float(np.sum(lp_distance(left_fd, right_fd, p=p) ** p))
+    return float(_v_asymptotic_stat_with_reps(*fd, weights=weights, p=p))
 
 
 def _anova_bootstrap(
-    fd_grouped: Tuple[FData, ...],
+    fd_grouped: Sequence[FData],
     n_reps: int,
     random_state: RandomStateLike = None,
     p: int = 2,
     equal_var: bool = True,
-) -> np.ndarray:
+) -> NDArrayFloat:
 
     n_groups = len(fd_grouped)
     if n_groups < 2:
@@ -186,13 +202,11 @@ def _anova_bootstrap(
                 "Domain range must match for every FData in fd_grouped.",
             )
 
-    start, stop = fd_grouped[0].domain_range[0]
-
     # List with sizes of each group
     sizes = [fd.n_samples for fd in fd_grouped]
 
     # Instance a random state object in case random_state is an int
-    random_state = check_random_state(random_state)
+    random_state = validate_random_state(random_state)
 
     if equal_var:
         k_est = concatenate(fd_grouped).cov().data_matrix[0, ..., 0]
@@ -201,34 +215,34 @@ def _anova_bootstrap(
         # Estimating covariances for each group
         k_est = [fd.cov().data_matrix[0, ..., 0] for fd in fd_grouped]
 
-    # Number of sample points for gaussian processes have to match
-    # the features of the covariances.
-    n_features = k_est[0].shape[0]
-
     # Simulating n_reps observations for each of the n_groups gaussian
     # processes
+    grid_points = getattr(fd_grouped[0], "grid_points", None)
+    if grid_points is None:
+        start, stop = fd_grouped[0].domain_range[0]
+        n_features = k_est[0].shape[0]
+        grid_points = np.linspace(start, stop, n_features)
+
     sim = [
-        make_gaussian_process(
+        make_gaussian(
             n_reps,
-            n_features=n_features,
-            start=start,
-            stop=stop,
+            grid_points=grid_points,
             cov=k_est[i],
             random_state=random_state,
         )
         for i in range(n_groups)
     ]
 
-    v_samples = np.empty(n_reps)
-    for i in range(n_reps):
-        fd = FDataGrid([s.data_matrix[i, ..., 0] for s in sim])
-        v_samples[i] = v_asymptotic_stat(fd, sizes, p=p)
-    return v_samples
+    return _v_asymptotic_stat_with_reps(*sim, weights=sizes, p=p)
+
+
+T = TypeVar("T", bound=FData)
 
 
 @overload
 def oneway_anova(
-    *args: FData,
+    first: T,
+    *rest: T,
     n_reps: int = 2000,
     return_dist: Literal[False] = False,
     random_state: RandomStateLike = None,
@@ -240,24 +254,26 @@ def oneway_anova(
 
 @overload
 def oneway_anova(
-    *args: FData,
+    first: T,
+    *rest: T,
     n_reps: int = 2000,
     return_dist: Literal[True],
     random_state: RandomStateLike = None,
     p: int = 2,
     equal_var: bool = True,
-) -> Tuple[float, float, np.ndarray]:
+) -> Tuple[float, float, NDArrayFloat]:
     pass
 
 
 def oneway_anova(
-    *args: FData,
+    first: T,
+    *rest: T,
     n_reps: int = 2000,
     return_dist: bool = False,
     random_state: RandomStateLike = None,
     p: int = 2,
     equal_var: bool = True,
-) -> Union[Tuple[float, float], Tuple[float, float, np.ndarray]]:
+) -> Tuple[float, float] | Tuple[float, float, NDArrayFloat]:
     r"""
     Perform one-way functional ANOVA.
 
@@ -289,7 +305,8 @@ def oneway_anova(
     This procedure is from Cuevas :footcite:`cuevas++_2004_anova`.
 
     Args:
-        args: The sample measurements for each each group.
+        first: First group of functions.
+        rest: Remaining groups.
         n_reps: Number of simulations for the bootstrap
             procedure. Defaults to 2000 (This value may change in future
             versions).
@@ -317,34 +334,27 @@ def oneway_anova(
         >>> fd = fetch_gait()["data"].coordinates[1]
         >>> fd1, fd2, fd3 = fd[:13], fd[13:26], fd[26:]
         >>> oneway_anova(fd1, fd2, fd3, random_state=RandomState(42))
-        (179.52499999999998, 0.5945)
+        (179.52499999999998, 0.56)
         >>> _, _, dist = oneway_anova(fd1, fd2, fd3, n_reps=3,
         ...     random_state=RandomState(42),
         ...     return_dist=True)
         >>> with printoptions(precision=4):
         ...     print(dist)
-        [ 184.0698 212.7395  195.3663]
+        [ 174.8663  202.1025  185.598 ]
 
     References:
         .. footbibliography::
 
     """
-    if len(args) < 2:
-        raise ValueError("At least two groups must be passed as parameter.")
-    if not all(isinstance(fd, FData) for fd in args):
-        raise ValueError("Argument type must inherit FData.")
     if n_reps < 1:
         raise ValueError("Number of simulations must be positive.")
 
-    fd_groups = args
-    if not all(isinstance(fd, type(fd_groups[0])) for fd in fd_groups[1:]):
-        raise TypeError('Found mixed FData types in arguments.')
-
-    for fd in fd_groups[1:]:
-        if not np.array_equal(fd.domain_range, fd_groups[0].domain_range):
+    for fd in rest:
+        if not np.array_equal(fd.domain_range, first.domain_range):
             raise ValueError("Domain range must match for every FData passed.")
 
-    if isinstance(fd_groups[0], FDataGrid):
+    fd_groups = [first, *rest]
+    if isinstance(first, FDataGrid):
         # Creating list with all the sample points
         list_sample = [fd.grid_points[0].tolist() for fd in fd_groups]
         # Checking that the all the entries in the list are the same
@@ -374,7 +384,7 @@ def oneway_anova(
         equal_var=equal_var,
     )
 
-    p_value = np.sum(simulation > vn) / len(simulation)
+    p_value = float(np.sum(simulation > vn) / len(simulation))
 
     if return_dist:
         return vn, p_value, simulation
