@@ -8,6 +8,7 @@ import numpy as np
 
 from ...misc._math import inner_product
 from ...representation.basis import Basis, FDataBasis
+from ...typing._numpy import NDArrayFloat
 
 CovariateType = TypeVar("CovariateType")
 
@@ -31,8 +32,8 @@ class CoefficientInfo(abc.ABC, Generic[CovariateType]):
     def regression_matrix(
         self,
         X: CovariateType,
-        y: np.ndarray,
-    ) -> np.ndarray:
+        y: NDArrayFloat,
+    ) -> NDArrayFloat:
         """
         Return the constant coefficients matrix for regression.
 
@@ -49,7 +50,7 @@ class CoefficientInfo(abc.ABC, Generic[CovariateType]):
     @abc.abstractmethod
     def convert_from_constant_coefs(
         self,
-        coefs: np.ndarray,
+        coefs: NDArrayFloat,
     ) -> CovariateType:
         """
         Return the coefficients object from the constant coefs.
@@ -68,7 +69,7 @@ class CoefficientInfo(abc.ABC, Generic[CovariateType]):
         self,
         coefs: CovariateType,
         X: CovariateType,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
         """
         Inner product.
 
@@ -79,28 +80,28 @@ class CoefficientInfo(abc.ABC, Generic[CovariateType]):
         pass
 
 
-class CoefficientInfoNdarray(CoefficientInfo[np.ndarray]):
+class CoefficientInfoNdarray(CoefficientInfo[NDArrayFloat]):
 
     def regression_matrix(  # noqa: D102
         self,
-        X: np.ndarray,
-        y: np.ndarray,
-    ) -> np.ndarray:
+        X: NDArrayFloat,
+        y: NDArrayFloat,
+    ) -> NDArrayFloat:
 
         return np.atleast_2d(X)
 
     def convert_from_constant_coefs(  # noqa: D102
         self,
-        coefs: np.ndarray,
-    ) -> np.ndarray:
+        coefs: NDArrayFloat,
+    ) -> NDArrayFloat:
 
         return coefs
 
     def inner_product(  # noqa: D102
         self,
-        coefs: np.ndarray,
-        X: np.ndarray,
-    ) -> np.ndarray:
+        coefs: NDArrayFloat,
+        X: NDArrayFloat,
+    ) -> NDArrayFloat:
 
         return inner_product(coefs, X)
 
@@ -117,8 +118,8 @@ class CoefficientInfoFDataBasis(CoefficientInfo[FDataBasis]):
     def regression_matrix(  # noqa: D102
         self,
         X: FDataBasis,
-        y: np.ndarray,
-    ) -> np.ndarray:
+        y: NDArrayFloat,
+    ) -> NDArrayFloat:
         # The matrix is the matrix of coefficients multiplied by
         # the matrix of inner products.
 
@@ -128,7 +129,7 @@ class CoefficientInfoFDataBasis(CoefficientInfo[FDataBasis]):
 
     def convert_from_constant_coefs(  # noqa: D102
         self,
-        coefs: np.ndarray,
+        coefs: NDArrayFloat,
     ) -> FDataBasis:
         return FDataBasis(self.basis.basis, coefs.T)
 
@@ -136,7 +137,7 @@ class CoefficientInfoFDataBasis(CoefficientInfo[FDataBasis]):
         self,
         coefs: FDataBasis,
         X: FDataBasis,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
         # Efficient implementation of the inner product using the
         # inner product matrix previously computed
         return inner_product(coefs, X, inner_product_matrix=self.inner_basis.T)
@@ -145,7 +146,7 @@ class CoefficientInfoFDataBasis(CoefficientInfo[FDataBasis]):
 @singledispatch
 def coefficient_info_from_covariate(
     X: CovariateType,
-    y: np.ndarray,
+    y: NDArrayFloat,
     **_: Any,
 ) -> CoefficientInfo[CovariateType]:
     """Make a coefficient info object from a covariate."""
@@ -153,18 +154,18 @@ def coefficient_info_from_covariate(
 
 
 @coefficient_info_from_covariate.register(np.ndarray)
-def _coefficient_info_from_covariate_ndarray(
-    X: np.ndarray,
-    y: np.ndarray,
+def _coefficient_info_from_covariate_ndarray(  # type: ignore[misc]
+    X: NDArrayFloat,
+    y: NDArrayFloat,
     **_: Any,
-) -> CoefficientInfo[np.ndarray]:
+) -> CoefficientInfo[NDArrayFloat]:
     return CoefficientInfoNdarray(basis=np.identity(X.shape[1], dtype=X.dtype))
 
 
 @coefficient_info_from_covariate.register(FDataBasis)
 def _coefficient_info_from_covariate_fdatabasis(
     X: FDataBasis,
-    y: np.ndarray,
+    y: NDArrayFloat,
     *,
     basis: Basis,
     **_: Any,
