@@ -6,7 +6,6 @@ from builtins import isinstance
 from typing import (
     TYPE_CHECKING,
     Any,
-    Iterable,
     Optional,
     Sequence,
     Type,
@@ -18,23 +17,16 @@ from typing import (
 import numpy as np
 import pandas.api.extensions
 
-from skfda._utils._utils import _to_array_maybe_ragged
-
-from ..._utils import _check_array_key, _int_to_real, constants
+from ..._utils import _check_array_key, _int_to_real, constants, nquad_vec
+from ...typing._base import DomainRange, GridPointsLike, LabelTupleLike
+from ...typing._numpy import ArrayLike, NDArrayBool, NDArrayFloat, NDArrayInt
 from .. import grid
 from .._functional_data import FData
-from .._typing import (
-    ArrayLike,
-    DomainRange,
-    GridPointsLike,
-    LabelTupleLike,
-    NDArrayInt,
-)
 from ..extrapolation import ExtrapolationLike
-from . import Basis
 
 if TYPE_CHECKING:
     from .. import FDataGrid
+    from . import Basis
 
 T = TypeVar('T', bound='FDataBasis')
 
@@ -73,13 +65,13 @@ class FDataBasis(FData):  # noqa: WPS214
             types of extrapolation.
 
     Examples:
-        >>> from skfda.representation.basis import FDataBasis, Monomial
+        >>> from skfda.representation.basis import FDataBasis, MonomialBasis
         >>>
-        >>> basis = Monomial(n_basis=4)
+        >>> basis = MonomialBasis(n_basis=4)
         >>> coefficients = [1, 1, 3, .5]
         >>> FDataBasis(basis, coefficients)
         FDataBasis(
-            basis=Monomial(domain_range=((0, 1),), n_basis=4),
+            basis=MonomialBasis(domain_range=((0.0, 1.0),), n_basis=4),
             coefficients=[[ 1.   1.   3.   0.5]],
             ...)
 
@@ -90,9 +82,7 @@ class FDataBasis(FData):  # noqa: WPS214
         basis: Basis,
         coefficients: ArrayLike,
         *,
-        dataset_label: Optional[str] = None,
         dataset_name: Optional[str] = None,
-        axes_labels: Optional[LabelTupleLike] = None,
         argument_names: Optional[LabelTupleLike] = None,
         coordinate_names: Optional[LabelTupleLike] = None,
         sample_names: Optional[LabelTupleLike] = None,
@@ -111,9 +101,7 @@ class FDataBasis(FData):  # noqa: WPS214
 
         super().__init__(
             extrapolation=extrapolation,
-            dataset_label=dataset_label,
             dataset_name=dataset_name,
-            axes_labels=axes_labels,
             argument_names=argument_names,
             coordinate_names=coordinate_names,
             sample_names=sample_names,
@@ -122,7 +110,7 @@ class FDataBasis(FData):  # noqa: WPS214
     @classmethod
     def from_data(
         cls,
-        data_matrix: np.ndarray,
+        data_matrix: Union[NDArrayFloat, NDArrayInt],
         *,
         basis: Basis,
         grid_points: Optional[GridPointsLike] = None,
@@ -188,8 +176,8 @@ class FDataBasis(FData):  # noqa: WPS214
             >>> x
             array([ 3.,  3.,  1.,  1.,  3.])
 
-            >>> from skfda.representation.basis import FDataBasis, Fourier
-            >>> basis = Fourier((0, 1), n_basis=3)
+            >>> from skfda.representation.basis import FDataBasis, FourierBasis
+            >>> basis = FourierBasis((0, 1), n_basis=3)
             >>> fd = FDataBasis.from_data(x, grid_points=t, basis=basis)
             >>> fd.coefficients.round(2)
             array([[ 2.  , 0.71, 0.71]])
@@ -212,13 +200,6 @@ class FDataBasis(FData):  # noqa: WPS214
             )
             grid_points = sample_points
 
-        # n is the samples
-        # m is the observations
-        # k is the number of elements of the basis
-
-        # Each sample in a column (m x n)
-        data_matrix = np.atleast_2d(data_matrix)
-
         fd = grid.FDataGrid(data_matrix=data_matrix, grid_points=grid_points)
 
         return fd.to_basis(basis=basis, method=method)
@@ -258,17 +239,17 @@ class FDataBasis(FData):  # noqa: WPS214
 
     def _evaluate(
         self,
-        eval_points: Union[ArrayLike, Iterable[ArrayLike]],
+        eval_points: NDArrayFloat,
         *,
         aligned: bool = True,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
 
         if aligned:
 
             eval_points = np.asarray(eval_points)
 
             # Each row contains the values of one element of the basis
-            basis_values = self.basis.evaluate(eval_points)
+            basis_values = self.basis(eval_points)
 
             res = np.tensordot(self.coefficients, basis_values, axes=(1, 0))
 
@@ -276,14 +257,12 @@ class FDataBasis(FData):  # noqa: WPS214
                 (self.n_samples, len(eval_points), self.dim_codomain),
             )
 
-        eval_points = cast(Iterable[ArrayLike], eval_points)
-
         res_list = [
-            np.sum((c * self.basis.evaluate(np.asarray(p)).T).T, axis=0)
+            np.sum((c * self.basis(np.asarray(p)).T).T, axis=0)
             for c, p in zip(self.coefficients, eval_points)
         ]
 
-        return _to_array_maybe_ragged(res_list)
+        return np.asarray(res_list)
 
     def shift(
         self,
@@ -356,6 +335,55 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return self.copy(basis=basis, coefficients=coefficients)
 
+    def integrate(
+        self: T,
+        *,
+        domain: Optional[DomainRange] = None,
+    ) -> NDArrayFloat:
+        """
+        Integration of the FData object.
+
+        The integration is performed over the whole domain. Thus, for a
+        function of several variables this will be a multiple integral.
+
+        For a vector valued function the vector of integrals will be
+        returned.
+
+        Args:
+            domain: Domain range where we want to integrate.
+                By default is None as we integrate on the whole domain.
+
+        Returns:
+            NumPy array of size (``n_samples``, ``dim_codomain``)
+            with the integrated data.
+
+        Examples:
+            We first create the data basis.
+                >>> from skfda.representation.basis import FDataBasis
+                >>> from skfda.representation.basis import MonomialBasis
+                >>> basis = MonomialBasis(n_basis=4)
+                >>> coefficients = [1, 1, 3, .5]
+                >>> fdata = FDataBasis(basis, coefficients)
+
+            Then we can integrate on the whole domain.
+                >>> fdata.integrate()
+                array([[ 2.625]])
+
+            Or we can do it on a given domain.
+                >>> fdata.integrate(domain=((0.5, 1),))
+                array([[ 1.8671875]])
+
+        """
+        if domain is None:
+            domain = self.basis.domain_range
+
+        integrated = nquad_vec(
+            self,
+            domain,
+        )
+
+        return integrated[:, 0, :]
+
     def sum(  # noqa: WPS125
         self: T,
         *,
@@ -382,12 +410,15 @@ class FDataBasis(FData):  # noqa: WPS214
             FDataBasis object.
 
         Examples:
-            >>> from skfda.representation.basis import FDataBasis, Monomial
-            >>> basis = Monomial(n_basis=4)
+            >>> from skfda.representation.basis import (
+            ...     FDataBasis,
+            ...     MonomialBasis,
+            ... )
+            >>> basis = MonomialBasis(n_basis=4)
             >>> coefficients = [[0.5, 1, 2, .5], [1.5, 1, 4, .5]]
             >>> FDataBasis(basis, coefficients).sum()
             FDataBasis(
-                basis=Monomial(domain_range=((0, 1),), n_basis=4),
+                basis=MonomialBasis(domain_range=((0.0, 1.0),), n_basis=4),
                 coefficients=[[ 2.  2.  6.  1.]],
                 ...)
 
@@ -409,28 +440,7 @@ class FDataBasis(FData):  # noqa: WPS214
             sample_names=(None,),
         )
 
-    def gmean(self: T, eval_points: Optional[np.ndarray] = None) -> T:
-        """Compute the geometric mean of the functional data object.
-
-        A numerical approach its used. The object its transformed into its
-        discrete representation and then the geometric mean is computed and
-        then the object is taken back to the basis representation.
-
-        Args:
-            eval_points: Set of points where the
-                functions are evaluated to obtain the discrete
-                representation of the object. If none are passed it calls
-                numpy.linspace with bounds equal to the ones defined in
-                self.domain_range and the number of points the maximum
-                between 501 and 10 times the number of basis.
-
-        Returns:
-            Geometric mean of the original object.
-
-        """
-        return self.to_grid(eval_points).gmean().to_basis(self.basis)
-
-    def var(self: T, eval_points: Optional[np.ndarray] = None) -> T:
+    def var(self: T, eval_points: Optional[NDArrayFloat] = None) -> T:
         """Compute the variance of the functional data object.
 
         A numerical approach its used. The object its transformed into its
@@ -451,7 +461,7 @@ class FDataBasis(FData):  # noqa: WPS214
         """
         return self.to_grid(eval_points).var().to_basis(self.basis)
 
-    def cov(self, eval_points: Optional[np.ndarray] = None) -> FData:
+    def cov(self, eval_points: Optional[NDArrayFloat] = None) -> FData:
         """Compute the covariance of the functional data object.
 
         A numerical approach its used. The object its transformed into its
@@ -495,9 +505,14 @@ class FDataBasis(FData):  # noqa: WPS214
               object.
 
         Examples:
-            >>> from skfda.representation.basis import FDataBasis, Monomial
-            >>> fd = FDataBasis(coefficients=[[1, 1, 1], [1, 0, 1]],
-            ...                 basis=Monomial(domain_range=(0,5), n_basis=3))
+            >>> from skfda.representation.basis import(
+            ...     FDataBasis,
+            ...     MonomialBasis,
+            ... )
+            >>> fd = FDataBasis(
+            ...     coefficients=[[1, 1, 1], [1, 0, 1]],
+            ...     basis=MonomialBasis(domain_range=(0,5), n_basis=3),
+            ... )
             >>> fd.to_grid([0, 1, 2])
             FDataGrid(
                 array([[[ 1.],
@@ -507,7 +522,7 @@ class FDataBasis(FData):  # noqa: WPS214
                         [ 2.],
                         [ 5.]]]),
                 grid_points=(array([ 0., 1., 2.]),),
-                domain_range=((0, 5),),
+                domain_range=((0.0, 5.0),),
                 ...)
 
         """
@@ -523,7 +538,7 @@ class FDataBasis(FData):  # noqa: WPS214
             grid_points = self._default_grid_points()
 
         return grid.FDataGrid(
-            self.evaluate(grid_points, grid=True),
+            self(grid_points, grid=True),
             grid_points=grid_points,
             domain_range=self.domain_range,
         )
@@ -531,7 +546,7 @@ class FDataBasis(FData):  # noqa: WPS214
     def to_basis(
         self,
         basis: Optional[Basis] = None,
-        eval_points: Optional[np.ndarray] = None,
+        eval_points: Optional[NDArrayFloat] = None,
         **kwargs: Any,
     ) -> FDataBasis:
         """
@@ -559,7 +574,7 @@ class FDataBasis(FData):  # noqa: WPS214
         *,
         deep: bool = False,  # For Pandas compatibility
         basis: Optional[Basis] = None,
-        coefficients: Optional[np.ndarray] = None,
+        coefficients: Optional[NDArrayFloat] = None,
         dataset_name: Optional[str] = None,
         argument_names: Optional[LabelTupleLike] = None,
         coordinate_names: Optional[LabelTupleLike] = None,
@@ -618,7 +633,7 @@ class FDataBasis(FData):  # noqa: WPS214
 
     def _array_to_R(  # noqa: N802
         self,
-        coefficients: np.ndarray,
+        coefficients: NDArrayFloat,
         transpose: bool = False,
     ) -> str:
         if len(coefficients.shape) == 1:
@@ -674,26 +689,12 @@ class FDataBasis(FData):  # noqa: WPS214
             and np.array_equal(self.coefficients, other.coefficients)
         )
 
-    def __eq__(self, other: object) -> np.ndarray:  # type: ignore[override]
+    def _eq_elemenwise(self: T, other: T) -> NDArrayBool:
         """Elementwise equality of FDataBasis."""
-        if not isinstance(other, type(self)) or self.dtype != other.dtype:
-            if other is pandas.NA:
-                return self.isna()
-            if pandas.api.types.is_list_like(other) and not isinstance(
-                other, (pandas.Series, pandas.Index, pandas.DataFrame),
-            ):
-                return np.concatenate([x == y for x, y in zip(self, other)])
-
-            return NotImplemented
-
-        if len(self) != len(other) and len(self) != 1 and len(other) != 1:
-            raise ValueError(
-                f"Different lengths: "
-                f"len(self)={len(self)} and "
-                f"len(other)={len(other)}",
-            )
-
-        return np.all(self.coefficients == other.coefficients, axis=1)
+        return np.all(  # type: ignore[no-any-return]
+            self.coefficients == other.coefficients,
+            axis=1,
+        )
 
     def concatenate(
         self: T,
@@ -743,7 +744,7 @@ class FDataBasis(FData):  # noqa: WPS214
         self,
         fd: FData,
         *,
-        eval_points: Optional[np.ndarray] = None,
+        eval_points: Optional[NDArrayFloat] = None,
         **kwargs: Any,
     ) -> FData:
         """
@@ -753,9 +754,9 @@ class FDataBasis(FData):  # noqa: WPS214
         compute the composition.
 
         Args:
-            fd (:class:`FData`): FData object to make the composition. Should
+            fd: FData object to make the composition. Should
                 have the same number of samples and image dimension equal to 1.
-            eval_points (array_like): Points to perform the evaluation.
+            eval_points: Points to perform the evaluation.
              kwargs: Named arguments to be passed to :func:`from_data`.
 
         Returns:
@@ -773,16 +774,22 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return composition
 
-    def __getitem__(self: T, key: Union[int, slice, np.ndarray]) -> T:
+    def __getitem__(
+        self: T,
+        key: Union[int, slice, NDArrayInt, NDArrayBool],
+    ) -> T:
         """Return self[key]."""
         key = _check_array_key(self.coefficients, key)
 
         return self.copy(
             coefficients=self.coefficients[key],
-            sample_names=np.array(self.sample_names)[key],
+            sample_names=list(np.array(self.sample_names)[key]),
         )
 
-    def __add__(self: T, other: Union[T, np.ndarray, float]) -> T:
+    def __add__(
+        self: T,
+        other: T,
+    ) -> T:
         """Addition for FDataBasis object."""
         if isinstance(other, FDataBasis) and self.basis == other.basis:
 
@@ -794,7 +801,10 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return NotImplemented
 
-    def __radd__(self: T, other: Union[T, np.ndarray, float]) -> T:
+    def __radd__(
+        self: T,
+        other: T,
+    ) -> T:
         """Addition for FDataBasis object."""
         if isinstance(other, FDataBasis) and self.basis == other.basis:
 
@@ -806,7 +816,10 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return NotImplemented
 
-    def __sub__(self: T, other: Union[T, np.ndarray, float]) -> T:
+    def __sub__(
+        self: T,
+        other: T,
+    ) -> T:
         """Subtraction for FDataBasis object."""
         if isinstance(other, FDataBasis) and self.basis == other.basis:
 
@@ -818,7 +831,10 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return NotImplemented
 
-    def __rsub__(self: T, other: Union[T, np.ndarray, float]) -> T:
+    def __rsub__(
+        self: T,
+        other: T,
+    ) -> T:
         """Right subtraction for FDataBasis object."""
         if isinstance(other, FDataBasis) and self.basis == other.basis:
 
@@ -830,7 +846,10 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return NotImplemented
 
-    def _mul_scalar(self: T, other: Union[np.ndarray, float]) -> T:
+    def _mul_scalar(
+        self: T,
+        other: Union[NDArrayFloat, NDArrayInt, float],
+    ) -> T:
         """Multiplication by scalar."""
         try:
             vector = np.atleast_1d(other)
@@ -848,15 +867,24 @@ class FDataBasis(FData):  # noqa: WPS214
             coefficients=self.coefficients * vector,
         )
 
-    def __mul__(self: T, other: Union[np.ndarray, float]) -> T:
+    def __mul__(
+        self: T,
+        other: Union[NDArrayFloat, NDArrayInt, float],
+    ) -> T:
         """Multiplication for FDataBasis object."""
         return self._mul_scalar(other)
 
-    def __rmul__(self: T, other: Union[np.ndarray, float]) -> T:
+    def __rmul__(
+        self: T,
+        other: Union[NDArrayFloat, NDArrayInt, float],
+    ) -> T:
         """Multiplication for FDataBasis object."""
         return self._mul_scalar(other)
 
-    def __truediv__(self: T, other: Union[np.ndarray, float]) -> T:
+    def __truediv__(
+        self: T,
+        other: Union[NDArrayFloat, NDArrayInt, float],
+    ) -> T:
         """Division for FDataBasis object."""
         try:
             other = 1 / np.asarray(other)
@@ -865,10 +893,17 @@ class FDataBasis(FData):  # noqa: WPS214
 
         return self._mul_scalar(other)
 
-    def __rtruediv__(self: T, other: Union[np.ndarray, float]) -> T:
+    def __rtruediv__(
+        self: T,
+        other: Union[NDArrayFloat, NDArrayInt, float],
+    ) -> T:
         """Right division for FDataBasis object."""
         return NotImplemented
 
+    def __neg__(self: T) -> T:
+        """Negation of FData object."""
+        return self.copy(coefficients=-self.coefficients)
+
     #####################################################################
     # Pandas ExtensionArray methods
     #####################################################################
@@ -905,17 +940,22 @@ class FDataBasis(FData):  # noqa: WPS214
         """
         return self.coefficients.nbytes
 
-    def isna(self) -> np.ndarray:
+    def isna(self) -> NDArrayBool:
         """
         Return a 1-D array indicating if each value is missing.
 
         Returns:
             na_values (np.ndarray): Positions of NA.
         """
-        return np.all(np.isnan(self.coefficients), axis=1)
+        return np.all(  # type: ignore[no-any-return]
+            np.isnan(self.coefficients),
+            axis=1,
+        )
 
 
-class FDataBasisDType(pandas.api.extensions.ExtensionDtype):  # type: ignore
+class FDataBasisDType(
+    pandas.api.extensions.ExtensionDtype,  # type: ignore[misc]
+):
     """DType corresponding to FDataBasis in Pandas."""
 
     kind = 'O'
